from mrakun import RakunDetector
from nltk.stem import WordNetLemmatizer
from nltk.corpus import stopwords
import glob
from random import randrange
import pytest


def test_basic_keywords1():

    hyperparameters = {
        "distance_threshold": 3,
        "num_keywords": 10,
        "pair_diff_length": 2,
        "distance_method": "editdistance",
        "stopwords": stopwords.words('english'),
        "bigram_count_threshold": 2,
        "num_tokens": [1]
    }

    keyword_detector = RakunDetector(hyperparameters)
    example_data = "./datasets/wiki20/docsutf8/7183.txt"
    keywords = keyword_detector.find_keywords(example_data)


def test_basic_visualization():

    hyperparameters = {
        "distance_threshold": 3,
        "num_keywords": 10,
        "pair_diff_length": 2,
        "stopwords": stopwords.words('english'),
        "distance_method": "editdistance",
        "bigram_count_threshold": 2,
        "num_tokens": [1]
    }

    keyword_detector = RakunDetector(hyperparameters)
    example_data = "./datasets/wiki20/docsutf8/7183.txt"
    keywords = keyword_detector.find_keywords(example_data)
    keyword_detector.visualize_network(display=False)


all_relevant = glob.glob("./datasets/wiki20/docsutf8/*")


@pytest.mark.parametrize("infile", all_relevant)
def test_basic_keywords2(infile):
    """
    A test across multiple files + parameter sets
    """

    for flx in all_relevant[0:10]:
        hyperparameters = {
            "distance_threshold": randrange(5),
            "num_keywords": randrange(10),
            "pair_diff_length": randrange(5),
            "distance_method": "editdistance",
            "stopwords": stopwords.words('english'),
            "bigram_count_threshold": randrange(5),
            "max_occurrence": randrange(10),
            "max_similar": randrange(4),
            "num_tokens": [1, 2, 3]
        }
        keyword_detector = RakunDetector(hyperparameters)
        keywords = keyword_detector.find_keywords(flx)



## a simple example for slovene news.

from mrakun import RakunDetector
from nltk.corpus import stopwords

#blob_of_text = "Slovenska nogometna reprezentanca se je s porazom 2:3 na gostovanju na Poljskem poslovila od kvalifikacij za evropsko prvenstvo 2020. Poljska je na razprodanem stadionu v Varšavi proslavila prestižno zmago v skupini G in četrto zaporedno uvrstitev na največje tekmovanje stare celine. Slovenija znova ni izpolnila cilja – kvalifikacije je končala na skromnem četrtem mestu, daleč od uvrstitve na veliko tekmovanje.Slovenija v Varšavi ni zgolj sklenila neuspešnih kvalifikacij, ampak je hkrati odprla priprave za ligo narodov 2020. V ta namen je selektor Matjaž Kek iskal odgovore o selekciji igralcev že danes na Poljskem, ko je na igrišče poslal premešano začetno postavo in osvežil različne položaje. V obrambi je kaznovanega štoperja Aljaža Struno zamenjal Miha Blažič, na levem boku pa je namesto Bojana Jokića zaigral Jure Balkovec. Napadalec Tim Matavž je iz udarne enajsterice izrinil Andraža Šporarja, priložnost od prve minute je na sredini igrišča dobil Jaka Bijol. Prenovljena izbrana vrsta se je tako kot v celotnih kvalifikacijah tudi v Varšavi predstavila z veliko nihanji.Začelo se je z neodgovornim prekrškom Kurtića za rumeni karton že po minuti igre in hitrim vodstvom Poljske po podaji iz kota. Nič ni pomagalo, da je slovenska vrsta protestirala, da je bil v vratarjevem prostoru storjen prekršek nad Janom Oblakom, ki je začutil bolečine v ramenu. Slovenija je po zaostanku poskušala igrati in izenačila po domiselni akciji Kurtića in Iličića, ki sta izigrala Poljake, da je Matavž hladnokrvno zadel za izenačenje. Nasprotnika sta se do konca polčasa izmenjavala v premoči, na stadionu pa je najbolj završalo pred odmorom, ko je na vnaprej dogovorjeno menjavo odšel poljski branilec Piszczek in se čustveno poslovil od nastopanja za reprezentanco.Tekma v Varšavi je minila brez posebnega tekmovalnega naboja na najvišji ravni. Poteza večera je uspela prvemu zvezdniku Poljske Robertu Lewandowskemu, ki je na začetku drugega polčasa zlahka preigral vso slovensko obrambo in zadel za drugo vodstvo domačih. Slovenija je popustila v disciplini, a se je po hitri akciji natančnih podaj in zadetku Iličića vrnila v igro. V odprtem boju za zmago so bili drznejši Poljaki, ki so v zmagoviti akciji izigrali gostujočo obrambo in se oddolžili navijačem za edini poraz kvalifikacij v Stožicah. Slovenija je tekmo končala z deseterico, saj je drugi rumeni karton prejel Kurtić, Keku pa se je ponesrečila menjava Bijola z Zajcem.»V Varšavi sem videl veliko dobrih stvari, a tudi veliko neodločnosti in neodgovornosti. Prelahko smo dopustili, da so Poljaki zabili tri gole. Posamezniki bodo morali razčistiti sami pri sebi. Preslabi smo bili v hitrih reakcijah. Škoda zapravljene točke,« je za nacionalno televizijo povedal selektor Matjaž Kek."

blob_of_text = "Prehrambni trendi se v zadnjih letih obračajo h koreninam. Vedno večja skupina ozaveščenih potrošnikov goji pričakovanja po naravnih, lokalno pridelanih in preprostih izdelkih, ki ne vsebujejo dodanih sladkorjev, barvil, arom ali drugih aditivov. Potrošniki se kot odgovor na svoj življenjski slog na trgu ozirajo po avtentičnih in naravnih živilih, strokovnjaki pa to obliko nakupnega vedenja opredeljujejo kot enega najpomembnejših prehranskih trendov v letu 2019. V tujini trend opisujejo z izrazom clean label oziroma čisto označevanje: »Jasno je, da gre za potrošniško zahtevo po čistih izdelkih, ki se nanaša na preglednost označb v smislu razumljivosti. Čedalje več potrošnikov si namreč želi uživati izdelke iz naravnih in enostavnih sestavin, ki jih razumejo in jim zaupajo. Clean label pomeni kratek seznam sestavin in njihovo razumljivost. Pomeni odsotnost umetnih dodatkov,« pojasnjujejo v Ljubljanskih mlekarnah.Razvoj izdelkov mora slediti zahtevam potrošnikov V Ljubljanskih mlekarnah so pred tremi leti pričeli s projektom clean label, v okviru katerega so med drugim prenovili vse sadne jogurte blagovne znamke Mu. Pred dobrim letom pa so storili še korak naprej in predstavili novo linijo izdelkov, jogurte Mu Natur. Njihova edinstvena prednost je, da so sestavljeni zgolj iz dveh sestavin – jogurta in sadja in so trenutno edini jogurti s sadjem brez dodanega sladkorja na slovenskem tržišču. Vendar pa je razvoj izdelkov po načelu manj je več zelo zahteven. Kot pojasnjujejo v Ljubljanskih mlekarnah, je bilo ključno vprašanje pri razvoju prvega tovrstnega izdelka, kako združiti samo dve sestavini, naravni jogurt in sadje brez vseh ostalih dodatkov, hkrati pa potrošnikom predstaviti kakovosten in okusen izdelek s primerno dolgim rokom uporabe. Da jim je to uspelo, je na koncu potrdila tudi Biotehniška fakulteta Univerze v Ljubljani s certifikatom 100 % naravno. Ugledni certifikat predstavlja jamstvo potrošnikom, da bodo zaužili pristne in kakovostne izdelke, ki vsebujejo le naravno prisotna sladkorja fruktozo in galaktozo, sestavini v sadju in mleku. Biotehniška fakulteta pa je vključena tudi pri sami proizvodnji jogurtov Mu Natur, in sicer v obliki rednega preverjanja ustreznosti izdelkov. Slovenke in Slovenci izbrali nov okus jogurtov Mu Natur breskev/mango Pozitiven odziv potrošnikov na jogurte Mu Natur je na Ljubljanskih mlekarnah botroval odločitvi, da na trgu predstavijo nov okus, ki se bo pridružil jagodi in borovnici. Pri razvoju okusa so želeli narediti korak naprej, zato so vanj vključili tudi potrošnike in jih povabili, da pomagajo pri sprejemu ključne odločitve v fazi razvoja izdelka – kakšen naj bo nov okus jogurta Mu Natur? Da bi preverili mnenje potrošnikov, so v Ljubljanskih mlekarnah konec avgusta v različnih slovenskih krajih organizirali degustacije, na katerih so imeli potrošniki možnost poskusiti tri okuse jogurta Mu Natur in glasovati za najljubšega. V sproščenem vzdušju so mladi in mladi po srcu okušali vzorce, med seboj delili mnenja in se zabavali ob stand-up vložkih komika Jana Kreuzerja z ekipo. »Pripravljenost ljudi, da nam pomagajo sprejeti odločitev glede novega okusa jogurta Mu Natur, je bila nad vsemi pričakovanji. To priložnost bi izkoristili, da se zahvalimo vsem sodelujočim in jim sporočimo, da smo njihovo mnenje seveda spoštovali. Največ glasov je prejel okus Mu Natur breskev/mango, ki je v teh dneh že na voljo na prodajnih mestih po Sloveniji,« so še sporočili iz Ljubljanskih mlekarn."

hyperparameters = {
    "distance_threshold": 5,
    "distance_method": "editdistance",
    "num_keywords": 20,
    "pair_diff_length": 2,
    "bigram_count_threshold": 1,
    "max_occurrence": 1,
    "max_similar": 1,
    "num_tokens": [1, 2, 3]
}

keyword_detector = RakunDetector(hyperparameters)
keywords = keyword_detector.find_keywords(blob_of_text, input_type="text")
print(keywords)
keyword_detector.default_visualization_parameters = {
    "top_n": 15,
    "max_node_size": 12,
    "min_node_size": 2,
    "label_font_size": 10,
    "text_color": "red",
    "num_layout_iterations": 100,
    "edge_width": 0.08,
    "alpha_channel": 0.5
}
#keyword_detector.visualize_network()



## multi term keyword example

from mrakun import RakunDetector
from nltk.corpus import stopwords
from nltk.stem import WordNetLemmatizer

lemmatizer = WordNetLemmatizer()

blob_of_text = "Brexit (/ˈbrɛksɪt, ˈbrɛɡzɪt/;[1] a portmanteau of \"British\" and \"exit\") is the scheduled withdrawal of the United Kingdom (UK) from the European Union (EU). Following a June 2016 referendum, in which 51.9% voted to leave, the UK government formally announced the country's withdrawal in March 2017, starting a two-year process that was due to conclude with the UK withdrawing on 29 March 2019. As the UK parliament thrice voted against the negotiated withdrawal agreement, that deadline has been extended twice, and is currently 31 October 2019.[2][3] An Act of Parliament requires the government to seek a third extension if no agreement is reached before 19 October. Withdrawal is advocated by Eurosceptics and opposed by pro-Europeanists, both of whom span the political spectrum. The UK joined the European Communities (EC) in 1973, with continued membership endorsed in a 1975 referendum. In the 1970s and 1980s, withdrawal from the EC was advocated mainly by the political left, e.g. in the Labour Party's 1983 election manifesto. From the 1990s, the eurosceptic wing of the Conservative Party grew, and led a rebellion over ratification of the 1992 Maastricht Treaty that established the EU. In parallel with the UK Independence Party (UKIP), and the cross-party People's Pledge campaign, it pressured Conservative Prime Minister David Cameron to hold a referendum on continued EU membership. Cameron, who had campaigned to remain, resigned after the result and was succeeded by Theresa May. On 29 March 2017, the UK government invoked Article 50 of the Treaty on European Union, formally starting the withdrawal. May called a snap general election in June 2017, which resulted in a Conservative minority government supported by the Democratic Unionist Party. UK–EU withdrawal negotiations began later that month. The UK negotiated to leave the EU customs union and single market. This resulted in the November 2018 withdrawal agreement, but the UK parliament voted against ratifying it three times. The Labour Party wanted any agreement to maintain a customs union, while many Conservatives opposed the agreement's financial settlement on the UK's share of EU financial obligations, as well as the Irish backstop designed to prevent border controls in Ireland. The Liberal Democrats, Scottish National Party and others seek to reverse Brexit through a second referendum. The EU has declined a re-negotiation that omits the backstop. In March 2019, the UK parliament voted for May to ask the EU to delay Brexit until October. Having failed to pass her agreement, May resigned as Prime Minister in July and was succeeded by Boris Johnson. He sought to replace parts of the agreement and vowed to leave the EU by the new deadline, with or without an agreement."

hyperparameters = {
    "distance_threshold": 4,
    "distance_method": "editdistance",
    "num_keywords": 10,
    "pair_diff_length": 3,
    "stopwords": stopwords.words('english'),
    "bigram_count_threshold": 0,
    "lemmatizer": lemmatizer,
    "num_tokens": [2]
}

keyword_detector = RakunDetector(hyperparameters)
keywords = keyword_detector.find_keywords(blob_of_text, input_type="text")
print(keywords)



## try to reproduce yake.
import yake
import pandas as pd
import tqdm
import glob
import os
dataset_list = glob.glob("../datasets/*")

os.system("rm -rvf ./results_for_datasets_yake")
os.system("rm -rvf ./eval_results/*")

num_keywords = 20


def read_contents(flx):
    return open(flx).read()


dnames = []
for dataset in dataset_list:
    dname = dataset.split("/")[-1]
    dnames.append(dname)

    directory = "results_for_datasets/{}".format(dname)
    if not os.path.exists(directory):
        os.makedirs(directory)
    fx = dataset + "/docsutf8/*"
    for kwf in tqdm.tqdm(glob.glob(fx)):
        text = read_contents(kwf)
        kw_extractor = yake.KeywordExtractor()
        keywords = kw_extractor.extract_keywords(text)[0:num_keywords]
        kwid = kwf.split("/")[-1].replace(".txt", "")
        kwx = directory + "/{}".format(kwid + ".key")
        btx = "\n".join(list(x[1] for x in keywords))
        with open(kwx, "w") as wx:
            wx.write(btx)

for name in dnames:
    cm1 = "python3 keyphrase2trec.py --datasetdir ../datasets/{} --results results_for_datasets/".format(
        name, name)
    os.system(cm1)
    command = "./trec_eval -m set_F -M1000  output/{}.qrel output/{}_results_for_datasets.out > eval_results/extracts_{}.scores".format(
        name, name, name)
    os.system(command)

### some processing of the data.
results = []
for j in glob.glob("eval_results/*"):
    try:
        dfx_rows = []
        with open(j) as fx:
            for line in fx:
                parts = line.strip().split()
                if len(parts) == 3:
                    try:
                        if parts[2] == ".":
                            continue
                        dfx_rows.append({
                            "score": parts[0],
                            "docID": parts[1],
                            "performance": float(parts[2])
                        })
                    except Exception as es:
                        print(es)

        dfx2 = pd.DataFrame(dfx_rows)
        dfx2['performance'] = dfx2['performance'].astype(float)
        dfx = dfx2.groupby(["score"])['performance'].mean()
        results.append({"dataset": j, "score": dfx['set_F']})

    except Exception as es:
        print(es)
pd.DataFrame(results).to_csv(
    "final_eval/final_{}_F_score.tsv".format(num_keywords), sep="\t")



import argparse
from os import path
from glob import glob
from string import punctuation

from tqdm import tqdm
from segtok.tokenizer import web_tokenizer, split_contractions


class Convert(object):
    def __init__(self, datasetdir, listoffilters):
        self.datasetdir = datasetdir
        self.datasetid = self.__get_datasetid__()
        self.lang = self.__get_language__()
        self.filters = self.__get_filters__(listoffilters)

    def build_result(self, results_dir):
        appname = self.__get_appname__(results_dir)
        conversor = self.__get_conversor__(appname)

        ptx = path.join(results_dir, self.datasetid, '*')

        listofresults = glob(ptx)

        toreturn = []
        for resultdoc in tqdm(sorted(listofresults), desc=appname, position=4):
            docid = self.__get_docid__(resultdoc)
            if docid not in self.qrels:
                print('[WARNING] Documento %s not fount in qrels' % docid)
                continue
            gt = self.qrels[docid]
            seen = set()
            result = []
            keyphrases = self.__readfile__(resultdoc).split('\n')
            if len(keyphrases) == 0:
                idkw = 'uk00'
                gt['--'] = (idkw, False)
            else:
                for weight, kw in conversor(keyphrases):
                    kw_key = self.__get_filtered_key__(kw)
                    if kw_key not in gt:
                        idkw = ('uk%d' % len(gt))
                        isrel = False
                        gt[kw] = (idkw, False)
                    else:
                        idkw, isrel = gt[kw_key]
                    if idkw not in seen:
                        seen.add(idkw)
                        result.append(idkw)
            self.qrels[docid] = gt
            toreturn.append((docid, result))
        return (appname, toreturn)

    def save_in_trec_format(self, output_path, appname, results):
        output_file = path.join(output_path,
                                "%s_%s.out" % (self.datasetid, appname))
        with open(output_file, 'w') as outfile:
            for (docid, result) in results:
                for i, instance in enumerate(result):
                    outfile.write("%s Q0 %s %d %d %s\n" %
                                  (docid, instance, (i + 1),
                                   (len(result) - i), appname))

    def save_qrel(self, output_path):
        output_file = path.join(output_path, "%s.qrel" % self.datasetid)
        with open(output_file, 'w') as outfile:
            for docid in self.qrels:
                for (idkw, isrel) in [
                    (idkw, isrel)
                        for (idkw, isrel) in self.qrels[docid].values()
                        if isrel
                ]:
                    outfile.write("%s\t0\t%s\t1\n" % (docid, idkw))

    def build_ground_truth(self):
        keysfiles = glob(path.join(self.datasetdir, 'keys', '*'))
        self.qrels = {}
        for keyfile in tqdm(keysfiles,
                            desc='Building Ground Truth (%s)' % self.datasetid,
                            position=2):
            docid = self.__get_docid__(keyfile)
            gt = {}
            keysunfiltered = self.__readfile__(keyfile).split('\n')
            for goldkey in keysunfiltered:
                gold_key = self.__get_filtered_key__(goldkey)
                if gold_key not in gt:
                    gt[gold_key] = ('k%d' % len(gt), True)
            self.qrels[docid] = gt
        return self.qrels

    # UTILS
    def __get_filters__(self, listoffilters):
        filters = []
        for filter_name in listoffilters:
            if filter_name == 'none':
                filters.append(self.__none_filter__)
            if filter_name == 'stem':
                if self.lang == 'polish':
                    from stems.polishstem import PolishStemmer
                    self.stem = PolishStemmer()
                    filters.append(self.__polish_stem__)
                elif self.lang == 'english':
                    from nltk.stem import PorterStemmer
                    self.stem = PorterStemmer()
                    filters.append(self.__nltk_stem__)
                elif self.lang == 'portuguese':
                    from nltk.stem import RSLPStemmer
                    self.stem = RSLPStemmer()
                    filters.append(self.__nltk_stem__)
                else:
                    from nltk.stem.snowball import SnowballStemmer
                    self.stem = SnowballStemmer(self.lang)
                    filters.append(self.__nltk_stem__)
        return filters

    def __get_filtered_key__(self, key):
        key_filtered = self.__simple_filter__(key)
        for termfilter in self.filters:
            key_filtered = termfilter(key_filtered)
        return key_filtered

    def __get_datasetid__(self):
        return path.split(path.realpath(self.datasetdir))[1]

    def __get_docid__(self, dockeypath):
        return path.basename(dockeypath).replace('.txt', '').replace(
            '.key', '').replace('.out', '').replace('.phrases', '')

    def __readfile__(self, filepath):
        with open(filepath, encoding='utf8') as infile:
            content = infile.read()
        return content

    def __get_language__(self):
        return self.__readfile__(path.join(self.datasetdir,
                                           'language.txt')).replace('\n', '')

    def __get_appname__(self, resultdir):
        return '_'.join([
            config for config in path.dirname(resultdir).split(path.sep)[-2:]
            if config != 'None'
        ])

    # FILTERS
    def __simple_filter__(self, word):
        term = word.lower()
        for p in punctuation:
            term = term.replace(p, ' ')
        term = ' '.join([w for w in split_contractions(web_tokenizer(term))])
        return term.strip()

    def __none_filter__(self, word):
        return word

    def __polish_stem__(self, word):
        return ' '.join(
            self.stem.stemmer_convert(
                [w for w in split_contractions(web_tokenizer(word))]))

    def __nltk_stem__(self, word):
        return ' '.join([
            self.stem.stem(w) for w in split_contractions(web_tokenizer(word))
        ])

    # CONVERSORS
    def __get_conversor__(self, method):
        if method.lower().startswith('rake') or method.lower().startswith(
                'yake') or method.lower().startswith(
                    'ibm') or method.lower().startswith('pke'):
            return self.__sorted_numericList__
        return self.__non_numericList__

    def __non_numericList__(self, listofkeys):
        return [(100. / (1. + i), kw) for i, kw in enumerate(listofkeys)
                if len(kw) > 0]

    def __sorted_numericList__(self, listofkeys):
        toreturn = []
        for key in listofkeys:
            parts = key.rsplit(' ', 1)
            if len(key) > 0 and len(parts) > 1:
                kw, weight = parts
                try:
                    weight = float(weight)
                except:
                    weight = 0.
                toreturn.append((weight, kw))
        return toreturn


parser = argparse.ArgumentParser()
required_args = parser.add_argument_group('required arguments')
required_args.add_argument('-d',
                           '--datasetdir',
                           type=str,
                           nargs='+',
                           help='',
                           required=True)
required_args.add_argument('-r',
                           '--results',
                           type=str,
                           nargs='+',
                           help='',
                           required=True)

parser.add_argument('-o',
                    '--output',
                    type=str,
                    nargs='?',
                    help='Output path.',
                    default='./output/')
parser.add_argument('-f',
                    '--filter',
                    type=str,
                    nargs='+',
                    help='Filter method.',
                    default=['none'],
                    choices=['none', 'stem'])

args = parser.parse_args()
for datasetdir in tqdm(args.datasetdir, position=1, desc='Datasets'):
    conv = Convert(datasetdir, listoffilters=args.filter)
    conv.build_ground_truth()

    for results in tqdm(args.results, position=3, desc=conv.datasetid):
        (appname, results_processed) = conv.build_result(results)
        conv.save_in_trec_format(args.output, appname, results_processed)
    conv.save_qrel(args.output)

# trec_eval -q -c -M1000 official_qrels submitted_results



from mrakun import RakunDetector
from nltk.stem import WordNetLemmatizer
from nltk.corpus import stopwords
from os import path

if not path.exists("../pretrained_models/fasttext/wiki.en.bin"):
    print(
        "Please, load a fasttext english pretrained model binary into folder ../pretrained_models."
    )

hyperparameters = {
    "distance_threshold": 0.2,
    "distance_method": "fasttext",
    "pretrained_embedding_path": '../pretrained_models/fasttext/wiki.en.bin',
    "num_keywords": 10,
    "pair_diff_length": 2,
    "stopwords": stopwords.words('english'),
    "bigram_count_threshold": 2,
    "lemmatizer": WordNetLemmatizer(),
    "num_tokens": [1]
}

keyword_detector = RakunDetector(hyperparameters)
example_data = "../datasets/wiki20/docsutf8/7183.txt"
keywords = keyword_detector.find_keywords(example_data)
print(keywords)
keyword_detector.visualize_network()



## a set of methods for evaluation of keyword quality.
import editdistance
from nltk.stem.porter import *
import numpy as np
from nltk.stem.porter import *


def compare_with_gold_fuzzy(detected_keywords,
                            gold_standard_keywords,
                            fuzzy_threshold=3,
                            keyword_separator=";"):
    """
    Fuzzy comparison of keyword matches. Given a fuzzy edit distance threshold, how many  keywords out of the top 10 are OK?
    input: detected_keywords (list of string).
    input: gold_standard_keywords (list of strings).
    input: fuzzy_threshold (int) -> max acceptable edit distance.    
    """

    precision_correct = 0
    precision_overall = 0

    recall_correct = 0
    recall_overall = 0

    for enx, keyword_set in enumerate(detected_keywords):

        gold_standard_set = gold_standard_keywords[enx]
        count = 0
        method_keywords = keyword_set.split(keyword_separator)

        if type(gold_standard_set) is float:  ## this is np.nan -> not defined.
            continue

        gold_standard_set = set(gold_standard_set.split(keyword_separator))

        top_n = len(gold_standard_set)
        if top_n >= len(method_keywords):
            top_n = len(method_keywords)

        ## recall
        parsed_rec = set()
        for el in method_keywords:
            if not el in parsed_rec:
                parsed_rec.add(el)
                if el in gold_standard_set:
                    recall_correct += 1
        recall_overall += top_n

        ## precision
        parsed_prec = set()
        for el in method_keywords:
            if not el in parsed_prec:
                parsed_prec.add(el)
                if el in gold_standard_set:
                    precision_correct += 1
        precision_overall += len(method_keywords)

    precision = float(precision_correct) / (
        precision_overall
    )  ## Number of correctly predicted over all predicted (num gold)

    recall = float(recall_correct) / (recall_overall
                                      )  ## Correct over all detected keywords

    if (precision + recall) > 0:
        F1 = 2 * (precision * recall) / (precision + recall)

    else:
        F1 = 0

    return precision, recall, F1


def compare_gold_exact(detected_keywords, gold_standard_keywords):

    stemmer = PorterStemmer()

    tp_fp_all = []
    tp_fn_all = []
    tp = []

    for enx, keyword_set in enumerate(detected_keywords):

        gold_standard_set = gold_standard_keywords[enx]

        method_keywords = list(set(keyword_set))[:10]
        gold_standard_set = list(set(gold_standard_set))

        # stemming
        stem_preds = []
        stem_true = []

        #print()
        #print("Preds: ", method_keywords)
        #print("True: ", gold_standard_set)

        for kw in method_keywords:
            kw = " ".join([stemmer.stem(word) for word in kw.split()])
            stem_preds.append(kw)

        for kw in gold_standard_set:
            kw = " ".join([stemmer.stem(word) for word in kw.split()])
            stem_true.append(kw)

        correct = 0

        for el in stem_preds:
            if el.lower() in stem_true:
                correct += 1

        tp_fn = float(len(gold_standard_set))
        tp_fp = float(len(method_keywords))

        tp_fn_all.append(tp_fn)
        tp_fp_all.append(tp_fp)

        tp.append(correct)

    precision = sum(tp) / sum(
        tp_fp_all
    )  ## Number of correctly predicted over all predicted (num gold)
    recall = sum(tp) / sum(tp_fn_all)  ## Correct over all detected keywords

    try:
        F1 = 2 * (precision * recall) / (precision + recall)
    except:
        F1 = 0

    return precision, recall, F1
